import GPLinearODEMaker.powers_of_negative_one

"""
qp_kernel function created by kernel_coder(). Requires 3 hyperparameters. Likely created using qp_kernel_base() as an input.
Use with include("src/kernels/qp_kernel.jl").
hyperparameters == ["qp_P", "se_λ", "p_amp"]
"""
function qp_kernel(
    hyperparameters::Vector{<:Real},
    δ::Real;
    dorder::Vector{<:Integer}=zeros(Int64, length(hyperparameters) + 2))

    @assert length(hyperparameters)==3 "hyperparameters is the wrong length"
    dorder_len = 5
    @assert length(dorder)==dorder_len "dorder is the wrong length"
    dorder2 = dorder[2]
    @assert maximum(dorder) < 3 "No more than two time derivatives for either t1 or t2 can be calculated"

    dorder[2] = sum(dorder[1:2])

    qp_P = hyperparameters[1]
    se_λ = hyperparameters[2]
    p_amp = hyperparameters[3]

    if view(dorder, 2:dorder_len)==[4, 0, 0, 2]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)^2 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^2 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2)) - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^4 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^4 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π^4*sin(δ*π/qp_P)^2/qp_P^4 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π^4*cos(δ*π/qp_P)^2/qp_P^4 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 + 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^4 + 320*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*sin(δ*π/qp_P)^4/qp_P^4 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^4*π^4*sin(δ*π/qp_P)^6/qp_P^4 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2 - 2880*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 4352*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^4*π^4*sin(δ*π/qp_P)^4*cos(δ*π/qp_P)^2/qp_P^4 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 1280*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^3 + 1024*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^4*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^5*cos(δ*π/qp_P)/qp_P^3 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^2 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^2 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 2]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^4 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^4 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 - 320*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^3 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^4*π^3*sin(δ*π/qp_P)^5*cos(δ*π/qp_P)/qp_P^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^2 + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 2]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*π^2*cos(δ*π/qp_P)^2/qp_P^2) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*p_amp*sin(δ*π/qp_P)^2 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 2]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 2]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*sin(δ*π/qp_P)^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*sin(δ*π/qp_P)^4
    end

    if view(dorder, 2:dorder_len)==[4, 0, 1, 1]
        func = 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)/se_λ^3 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/se_λ^3 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2/se_λ^3 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^2/se_λ^3 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^2/se_λ^3 - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/se_λ^3 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π^4*sin(δ*π/qp_P)^2/(qp_P^4*se_λ^3) + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π^4*cos(δ*π/qp_P)^2/(qp_P^4*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^3*π^4*sin(δ*π/qp_P)^4/(qp_P^4*se_λ^3) - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/se_λ^3 - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 576*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^3*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/(qp_P^4*se_λ^3) - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^3*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 1, 1]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ/se_λ^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/se_λ^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2/se_λ^3 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2/se_λ^3 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/se_λ^3 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^3*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 1, 1]
        func = -8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*sin(δ*π/qp_P)^2/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ^2/se_λ^3 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/se_λ^3 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/se_λ^3 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2/se_λ^3 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3) - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 1, 1]
        func = -8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*sin(δ*π/qp_P)^2/se_λ^3 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/se_λ^3 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 1, 1]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp*sin(δ*π/qp_P)^2/se_λ^3
    end

    if view(dorder, 2:dorder_len)==[4, 1, 0, 1]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-16*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 16*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 32*δ*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-16*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 16*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 32*δ*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*sin(δ*π/qp_P)^2 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^4*sin(δ*π/qp_P)^2/qp_P^5 - 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^4*cos(δ*π/qp_P)^2/qp_P^5 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^4*sin(δ*π/qp_P)^4/qp_P^5 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 + 1792*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 - 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^4 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^3 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^6 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/qp_P^6 - 1024*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^6 - 2304*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^5*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)^3/qp_P^6 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^5*π^5*sin(δ*π/qp_P)^5*cos(δ*π/qp_P)/qp_P^6 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/qp_P^5 - 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/qp_P^5 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^4/qp_P^5 - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 1280*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 - 1024*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^5*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^4*cos(δ*π/qp_P)^2/qp_P^5 - 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 - 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[3, 1, 0, 1]
        func = 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3) + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-16*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 16*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 32*δ*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2)) - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2 - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4 + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^4 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^4*sin(δ*π/qp_P)^2/qp_P^5 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π^4*cos(δ*π/qp_P)^2/qp_P^5 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^4*sin(δ*π/qp_P)^4/qp_P^5 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P + 320*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^5*π^4*sin(δ*π/qp_P)^4*cos(δ*π/qp_P)^2/qp_P^5 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[2, 1, 0, 1]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-16*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 16*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 32*δ*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*sin(δ*π/qp_P)^2 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*δ*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[1, 1, 0, 1]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 8*δ*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*δ*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^3) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*sin(δ*π/qp_P)^2 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[0, 1, 0, 1]
        func = 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^3*π*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 0, 1]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2)) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^2 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^2 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^4*sin(δ*π/qp_P)^2/qp_P^4 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^4*cos(δ*π/qp_P)^2/qp_P^4 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^4*sin(δ*π/qp_P)^4/qp_P^4 - 576*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^3 - 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 1]
        func = 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^3*π^3*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 1]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(8*p_amp*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 8*p_amp*π^2*cos(δ*π/qp_P)^2/qp_P^2) - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 1]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 1]
        func = -4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp*sin(δ*π/qp_P)^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 2, 0]
        func = 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)/se_λ^6 - 36*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^4 - 36*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^4 + 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^4 + 72*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^6 + 72*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^6 - 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4/se_λ^4 - 9*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2/se_λ^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^6 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4/se_λ^6 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2/se_λ^6 - 72*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^4 - 18*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^4 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^6 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/(qp_P^4*se_λ^4) - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/(qp_P^4*se_λ^4) - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*p_amp^2*π^4*sin(δ*π/qp_P)^2/(qp_P^4*se_λ^6) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*p_amp^2*π^4*cos(δ*π/qp_P)^2/(qp_P^4*se_λ^6) - 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^4) + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^6) - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^4) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^6)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 2, 0]
        func = -18*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^4 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ/se_λ^6 - 18*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^4 - 18*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^4 + 36*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6 - 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^4 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^6 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^6 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^6 - 9*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^4 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^4) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^6)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 2, 0]
        func = -6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)/se_λ^4 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2/se_λ^6 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^4 - 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^4 - 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^4 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^6 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^6
    end

    if view(dorder, 2:dorder_len)==[1, 0, 2, 0]
        func = -6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ/se_λ^4 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3/se_λ^6 - 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^4 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^6
    end

    if view(dorder, 2:dorder_len)==[0, 0, 2, 0]
        func = -3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2/se_λ^4 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^4/se_λ^6
    end

    if view(dorder, 2:dorder_len)==[4, 1, 1, 0]
        func = 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)/se_λ^3 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/(qp_P^5*se_λ^3) - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) - 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^4*se_λ^3) + 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^6*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^4*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/(qp_P^6*se_λ^3) - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^4*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/(qp_P^6*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/(qp_P^5*se_λ^3) - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^4*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^4*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[3, 1, 1, 0]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)/se_λ^3 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^4*sin(δ*π/qp_P)^2/(qp_P^5*se_λ^3) - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π^4*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^4*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^4*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/(qp_P^5*se_λ^3) + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[2, 1, 1, 0]
        func = 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)/se_λ^3 + 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[1, 1, 1, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)/se_λ^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[0, 1, 1, 0]
        func = 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^3*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^2*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[4, 0, 1, 0]
        func = 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2/se_λ^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/(qp_P^4*se_λ^3) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/(qp_P^4*se_λ^3) + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 1, 0]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3/se_λ^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/(qp_P^3*se_λ^3)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 1, 0]
        func = 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)/se_λ^3 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))/se_λ^3
    end

    if view(dorder, 2:dorder_len)==[1, 0, 1, 0]
        func = 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ/se_λ^3 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)/se_λ^3
    end

    if view(dorder, 2:dorder_len)==[0, 0, 1, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2/se_λ^3
    end

    if view(dorder, 2:dorder_len)==[4, 2, 0, 0]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)^2 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2)) + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(24*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 24*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 - 16*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 16*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 + 96*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(24*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 24*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 - 16*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 16*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 + 96*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) - 320*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 320*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^6*sin(δ*π/qp_P)^2/qp_P^8 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^6*cos(δ*π/qp_P)^2/qp_P^8 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^6*sin(δ*π/qp_P)^4/qp_P^8 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^6*cos(δ*π/qp_P)^4/qp_P^8 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5 - 384*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 640*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^7 - 640*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/qp_P^7 + 640*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^7 - 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/qp_P^6 + 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/qp_P^6 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^4*sin(δ*π/qp_P)^2/qp_P^5 - 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^4*cos(δ*π/qp_P)^2/qp_P^5 + 640*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^6*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^8 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^6*π^6*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^4/qp_P^8 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^6*π^6*sin(δ*π/qp_P)^4*cos(δ*π/qp_P)^2/qp_P^8 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*cos(δ*π/qp_P)^2/qp_P^4 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^2/qp_P^4 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*cos(δ*π/qp_P)^2/qp_P^4 - 2048*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^6 + 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^5*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^7 - 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^5*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/qp_P^7 + 768*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^5*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^7 + 1024*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^6*π^5*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)^3/qp_P^7 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/qp_P^4 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 32*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 96*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[3, 2, 0, 0]
        func = 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(24*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 24*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 - 16*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 16*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 + 96*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5 - 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 128*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 - 512*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^6 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^7 - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/qp_P^7 + 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^7 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^6*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)^3/qp_P^7 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*cos(δ*π/qp_P)^2/qp_P^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/qp_P^4 - 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 - 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4
    end

    if view(dorder, 2:dorder_len)==[2, 2, 0, 0]
        func = 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)^2 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(24*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 24*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 - 16*δ^2*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^6 + 16*δ^2*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^6 + 96*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*cos(δ*π/qp_P)^2/qp_P^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[1, 2, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 16*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 + 16*δ^2*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^5) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/qp_P^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4
    end

    if view(dorder, 2:dorder_len)==[0, 2, 0, 0]
        func = 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^4 - 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^4 - 8*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ^2*p_amp^4*π^2*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^4
    end

    if view(dorder, 2:dorder_len)==[4, 1, 0, 0]
        func = 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^5 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^5 - 192*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^6 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^5*sin(δ*π/qp_P)*cos(δ*π/qp_P)^3/qp_P^6 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^5*sin(δ*π/qp_P)^3*cos(δ*π/qp_P)/qp_P^6 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2/qp_P^5 - 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*cos(δ*π/qp_P)^2/qp_P^5 + 256*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^4*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 24*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[3, 1, 0, 0]
        func = 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2)) - 48*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^5 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^5 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^4*π^4*sin(δ*π/qp_P)^2*cos(δ*π/qp_P)^2/qp_P^5 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 12*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[2, 1, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-8*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 8*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3 - 16*δ*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^4) + 2*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[1, 1, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2 - 4*δ*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^3 + 4*δ*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^3) + 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[0, 1, 0, 0]
        func = 4*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*δ*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^4 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))^2 + 6*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 - 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*sin(δ*π/qp_P)^2/qp_P^4 + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^4*cos(δ*π/qp_P)^2/qp_P^4 + 64*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^3 + 3*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P) + 16*exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*p_amp^2*π^3*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P^3
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)^2 + exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(4*p_amp^2*π^2*sin(δ*π/qp_P)^2/qp_P^2 - 4*p_amp^2*π^2*cos(δ*π/qp_P)^2/qp_P^2 - se_λ^(-2))
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)*(-δ/se_λ^2 - 4*p_amp^2*π*sin(δ*π/qp_P)*cos(δ*π/qp_P)/qp_P)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 0]
        func = exp(-2*p_amp^2*sin(δ*π/qp_P)^2 + (-1/2)*δ^2/se_λ^2)
    end

    dorder[2] = dorder2  # resetting dorder[2]
    return powers_of_negative_one(dorder2) * float(func)  # correcting for amount of t2 derivatives

end


return qp_kernel, 3  # the function handle and the number of kernel hyperparameters
