import GPLinearODEMaker.powers_of_negative_one

"""
rq_kernel function created by kernel_coder(). Requires 3 hyperparameters. Likely created using rq_kernel_base() as an input.
Use with include("src/kernels/rq_kernel.jl").
hyperparameters == ["α", "λ", "rq_P"]
"""
function rq_kernel(
    hyperparameters::Vector{<:Real},
    δ::Real;
    dorder::Vector{<:Integer}=zeros(Int64, length(hyperparameters) + 2))

    @assert length(hyperparameters)==3 "hyperparameters is the wrong length"
    dorder_len = 5
    @assert length(dorder)==dorder_len "dorder is the wrong length"
    dorder2 = dorder[2]
    @assert maximum(dorder) < 3 "No more than two time derivatives for either t1 or t2 can be calculated"

    dorder[2] = sum(dorder[1:2])

    α = hyperparameters[1]
    λ = hyperparameters[2]
    rq_P = hyperparameters[3]

    if view(dorder, 2:dorder_len)==[4, 0, 0, 2]
        func = -320*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 320*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 64*δ^2*λ*π^6*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^8 - 64*δ^2*λ*π^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^8 - 960*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 960*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 640*δ*λ*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^7 + 7040*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*δ^2*λ^2*(-1 - α)*π^6*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) + 960*δ^2*λ^2*(-1 - α)*π^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) - 7680*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 7680*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 9600*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 9600*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) + 960*δ^2*λ^3*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 960*δ^2*λ^3*(-2 - α)*(-1 - α)*π^6*cos(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 6016*δ^2*λ^2*(-1 - α)*π^6*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) - 5120*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 9600*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 44800*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 9600*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 22080*δ^2*λ^3*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 22080*δ^2*λ^3*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 25600*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) + 25600*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) - 11520*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) + 43520*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) - 11520*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) - 10240*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^7) - 15360*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^8) + 15360*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^6*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^8) - 4096*δ^2*λ^6*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*π^6*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^8)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 2]
        func = -128*δ*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 128*δ*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 192*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*δ^2*λ*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^7 - 576*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 576*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 384*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 384*δ*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 768*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 2816*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*δ^2*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 960*δ^2*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 3072*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 3072*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 960*δ^2*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 4480*δ^2*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 960*δ^2*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 2048*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) + 2560*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) - 1024*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^7)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 2]
        func = 24*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 24*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*δ^2*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 16*δ^2*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 96*δ*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 96*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*δ^2*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 48*δ^2*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 288*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 288*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 352*δ^2*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 384*δ*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*δ^2*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 384*δ^2*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 256*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 2]
        func = 16*δ*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*δ*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 8*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*δ^2*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*δ*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*δ^2*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*δ^2*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 64*δ^2*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 2]
        func = 4*δ^2*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 4*δ^2*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 8*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*δ^2*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[4, 0, 1, 1]
        func = 64*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 + 64*δ*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 + 512*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*λ*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 2944*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 4992*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 7424*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1920*δ*λ*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 2048*δ*λ*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 7168*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 3072*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 2880*δ*λ^2*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 15360*δ*λ^2*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 4800*δ*λ^2*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 2048*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^5) + 12160*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 19200*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1920*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^7*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 10240*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 5120*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^7*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) + 2048*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*π^5*sin(δ*π/rq_P)^7*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^6)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 1, 1]
        func = -48*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*δ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*δ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 + 288*λ*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 384*λ*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 128*δ*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 96*δ*λ*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 864*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 288*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 96*δ*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 736*δ*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1248*δ*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1856*δ*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1792*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 768*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 512*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^5)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 1, 1]
        func = -8*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*δ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 80*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 64*λ^2*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*δ*λ*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 128*δ*λ*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 288*δ*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*δ*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 128*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 1, 1]
        func = 4*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*δ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*δ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*λ*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 8*δ*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 40*δ*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 32*δ*λ^2*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 1, 1]
        func = 4*δ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 8*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[4, 1, 0, 1]
        func = -192*λ^2*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 192*λ^2*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*λ*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 192*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 192*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) + 1408*λ^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 1408*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 1536*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1536*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1536*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 48*λ^2*(-1 - α)*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 896*δ*λ^2*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 896*δ*λ^2*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 64*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 - 3072*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) + 3072*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) - 1024*λ^4*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1024*λ^4*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1024*λ^4*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1408*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 352*λ^2*(-1 - α)*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 896*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^6) - 896*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^6) - 960*δ*λ^3*(-2 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 4480*δ*λ^3*(-2 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 960*δ*λ^3*(-2 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 768*δ*λ^3*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 3072*δ*λ^3*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 768*δ*λ^3*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^5) + 1536*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1728*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) + 7552*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) - 1728*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) - 2560*δ*λ^4*(-3 - α)*(-2 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 2560*δ*λ^4*(-3 - α)*(-2 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*δ*λ^4*(-3 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 2560*δ*λ^4*(-3 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1024*δ*λ^4*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*δ*λ^4*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 896*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 896*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 1024*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 6144*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^6) + 6144*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^6) - 1024*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 1024*δ*λ^5*(-4 - α)*(-3 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 1024*δ*λ^5*(-4 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) + 768*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 768*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 1408*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^5*rq_P^6) + 1024*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1024*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1536*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1536*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6)
    end

    if view(dorder, 2:dorder_len)==[3, 1, 0, 1]
        func = -144*λ^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 144*λ^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*δ*λ^2*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 48*δ*λ^2*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 48*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*π^3*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 144*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 144*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 192*λ^3*(-2 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 192*λ^3*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 48*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 48*δ*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) + 288*δ*λ^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 384*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 144*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 144*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^3*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*π^3*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 288*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 384*δ*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 384*δ*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*δ*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*δ*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 48*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 64*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 576*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) + 576*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) - 256*δ*λ^4*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 256*δ*λ^4*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 288*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 512*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^5) + 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 256*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5)
    end

    if view(dorder, 2:dorder_len)==[2, 1, 0, 1]
        func = -8*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*λ*π^2*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*π^2*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 32*λ^2*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 32*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 32*δ*λ^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 32*δ*λ^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 32*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 16*λ^2*(-1 - α)*π^2*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 32*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 32*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 64*δ*λ^3*(-2 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 64*δ*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 32*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 32*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 64*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[1, 1, 0, 1]
        func = -4*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*π*(4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P + 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[0, 1, 0, 1]
        func = 8*δ*λ^2*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) + 4*δ*λ*π*(2*λ*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 0, 1]
        func = 64*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 + 192*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 64*δ*λ*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^6 - 1408*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 1536*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 960*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*δ*λ^2*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 1024*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 960*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 4480*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 960*δ*λ^3*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 2560*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^5*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^5*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 1]
        func = 16*δ*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*δ*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^5 - 48*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 144*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 144*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 48*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*δ*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 352*δ*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*δ*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 256*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 1]
        func = -8*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*δ*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 32*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*δ*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 64*δ*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 1]
        func = -4*δ*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*δ*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 16*δ*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 1]
        func = 4*δ*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 2, 0]
        func = -160*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 448*λ*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 192*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^8*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 2688*λ*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 5184*λ*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 7872*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 6016*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 5632*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 1536*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^8*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) - 1024*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*π^4*sin(δ*π/rq_P)^8*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 2, 0]
        func = -96*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 160*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 768*λ*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 448*λ*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 960*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) + 192*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^7*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) - 256*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^7*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 2, 0]
        func = 16*(-1 - α)*π^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 48*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 16*λ*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) - 144*λ*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) - 64*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 2, 0]
        func = -16*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P) - 16*λ*(-2 - α)*(-1 - α)*π*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 2, 0]
        func = -4*(-1 - α)*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/α
    end

    if view(dorder, 2:dorder_len)==[4, 1, 1, 0]
        func = -16*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 96*λ*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*λ*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*λ*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 96*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*λ*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*λ^2*(-2 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 704*λ*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 704*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 1248*λ^2*(-2 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1856*λ^2*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1152*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1152*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 2400*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 3008*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1792*λ^3*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*λ^3*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1792*λ^3*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*λ^3*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1024*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 96*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 704*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*λ^2*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 4608*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) + 512*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 512*λ^4*(-4 - α)*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 512*λ^4*(-4 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) - 1152*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1152*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 704*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*λ^3*(-2 - α)*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1536*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^5*rq_P^4) - 1024*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 768*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 1, 1, 0]
        func = 16*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 96*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 96*λ*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 16*λ*π^3*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 96*λ*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 96*λ*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 288*λ^2*(-2 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 96*λ^2*(-2 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 192*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 480*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 96*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) + 128*λ^3*(-3 - α)*(-2 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) + 128*λ^3*(-3 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*λ^2*(-1 - α)*π^3*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*π^3*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 256*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^3) - 192*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 96*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 64*λ^3*(-2 - α)*(-1 - α)*π^3*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 128*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 1, 1, 0]
        func = 4*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 4*λ*π^2*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*π^2*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 32*λ*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) + 32*λ^2*(-2 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) + 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*λ^2*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^2) - 32*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 16*λ^2*(-1 - α)*π^2*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 32*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 1, 1, 0]
        func = -4*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P - 4*λ*π*(-2*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P - 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P)
    end

    if view(dorder, 2:dorder_len)==[0, 1, 1, 0]
        func = -2*(2*λ*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α) - 4*λ*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/α^2
    end

    if view(dorder, 2:dorder_len)==[4, 0, 1, 0]
        func = -16*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 128*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*λ*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 736*λ*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 1248*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1856*λ^2*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1792*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 768*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^6*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 1, 0]
        func = 16*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 96*λ*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 128*λ*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 288*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*λ^2*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 128*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^5*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 1, 0]
        func = 4*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 8*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 40*λ*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 32*λ^2*(-2 - α)*(-1 - α)*π^2*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 1, 0]
        func = -4*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P - 8*λ*(-1 - α)*π*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P)
    end

    if view(dorder, 2:dorder_len)==[0, 0, 1, 0]
        func = -2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)
    end

    if view(dorder, 2:dorder_len)==[4, 2, 0, 0]
        func = -96*λ^2*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 96*λ^2*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 16*λ*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 96*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) - 96*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) + 704*λ^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 768*λ^3*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 768*λ^3*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 704*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) - 1536*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1536*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 1536*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1536*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 512*λ^4*(-3 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*λ^4*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*λ^4*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 704*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 2304*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^4) + 2304*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^4) - 1536*λ^4*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*λ^4*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*λ^4*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 704*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 768*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 768*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 768*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 768*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 352*λ^2*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 3072*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^5*rq_P^4) + 1536*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 1536*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 512*λ^4*(-3 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 512*λ^4*(-3 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 512*λ^4*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 384*λ^3*(-2 - α)*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1536*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 2, 0, 0]
        func = -96*λ^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 96*λ^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 128*λ^3*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 16*λ*π^3*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 96*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^3) + 96*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^3) - 256*λ^3*(-2 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 256*λ^3*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 384*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^3) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 128*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 128*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 48*λ^2*(-1 - α)*π^3*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*π^3*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 256*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 64*λ^3*(-2 - α)*(-1 - α)*π^3*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 64*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 2, 0, 0]
        func = 4*λ*π^2*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*π^2*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 32*λ^2*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 32*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^2) + 32*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 16*λ^2*(-1 - α)*π^2*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 16*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 2, 0, 0]
        func = -4*λ*π*(4*λ*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)))*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if view(dorder, 2:dorder_len)==[0, 2, 0, 0]
        func = (2*λ*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-α) + 4*λ^2*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/α^3
    end

    if view(dorder, 2:dorder_len)==[4, 1, 0, 0]
        func = 48*λ^2*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 48*λ^2*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 48*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 48*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 352*λ^2*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 352*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*λ^3*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*λ^3*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 768*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 256*λ^4*(-3 - α)*(-2 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 256*λ^4*(-3 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 256*λ^4*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 352*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 384*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 1, 0, 0]
        func = 48*λ^2*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*λ^2*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 + 48*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 48*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 64*λ^3*(-2 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 64*λ^3*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 128*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 64*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 1, 0, 0]
        func = 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 + 16*λ^2*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 16*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 16*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 1, 0, 0]
        func = -4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if view(dorder, 2:dorder_len)==[0, 1, 0, 0]
        func = (2*λ*sin(δ*π/rq_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_P)^2/α))*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-α)
    end

    if view(dorder, 2:dorder_len)==[4, 0, 0, 0]
        func = -16*λ*π^4*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*λ*π^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^4 - 48*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*λ^2*(-1 - α)*π^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*λ^2*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 384*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*λ^3*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 256*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*π^4*sin(δ*π/rq_P)^4*cos(δ*π/rq_P)^4*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if view(dorder, 2:dorder_len)==[3, 0, 0, 0]
        func = 16*λ*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^3 - 48*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*λ^2*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 64*λ^3*(-2 - α)*(-1 - α)*π^3*sin(δ*π/rq_P)^3*cos(δ*π/rq_P)^3*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if view(dorder, 2:dorder_len)==[2, 0, 0, 0]
        func = 4*λ*π^2*sin(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*λ*π^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P^2 - 16*λ^2*(-1 - α)*π^2*sin(δ*π/rq_P)^2*cos(δ*π/rq_P)^2*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if view(dorder, 2:dorder_len)==[1, 0, 0, 0]
        func = -4*λ*π*sin(δ*π/rq_P)*cos(δ*π/rq_P)*(1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if view(dorder, 2:dorder_len)==[0, 0, 0, 0]
        func = (1 + 2*λ*sin(δ*π/rq_P)^2/α)^(-α)
    end

    dorder[2] = dorder2  # resetting dorder[2]
    return powers_of_negative_one(dorder2) * float(func)  # correcting for amount of t2 derivatives

end


return rq_kernel, 3  # the function handle and the number of kernel hyperparameters
