

"""
rq_periodic_kernel function created by kernel_coder(). Requires 3 hyperparameters. Likely created using rq_periodic_kernel_base() as an input. 
Use with include("src/kernels/rq_periodic_kernel.jl").
hyperparameters == ["α", "λ", "rq_P"]
"""
function rq_periodic_kernel(
    hyperparameters::Vector{<:Real}, 
    δ::Real; 
    dorder::Vector{<:Integer}=zeros(Int64, length(hyperparameters) + 2))

    @assert length(hyperparameters)==3 "hyperparameters is the wrong length"
    @assert length(dorder)==(3 + 2) "dorder is the wrong length"
    even_time_derivative = powers_of_negative_one(dorder[2])
    @assert maximum(dorder) < 3 "No more than two time derivatives for either t1 or t2 can be calculated"

    dorder = append!([sum(dorder[1:2])], dorder[3:end])

    α = hyperparameters[1]
    λ = hyperparameters[2]
    rq_P = hyperparameters[3]

    if dorder==[4, 0, 0, 2]
        func = -320*pi^4*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 320*pi^4*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 64*pi^6*δ^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^8 - 64*pi^6*δ^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^8 - 960*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 960*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 640*pi^5*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^7 + 7040*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*pi^6*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) + 960*pi^6*δ^2*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) - 7680*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 7680*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 9600*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 9600*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) + 960*pi^6*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 960*pi^6*δ^2*λ^3*(-2 - α)*(-1 - α)*cos(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 6016*pi^6*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^8) - 5120*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 9600*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 44800*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 9600*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 22080*pi^6*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 22080*pi^6*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^8) - 25600*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) + 25600*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) - 11520*pi^6*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) + 43520*pi^6*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) - 11520*pi^6*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^8) - 10240*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^7) - 15360*pi^6*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^8) + 15360*pi^6*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^8) - 4096*pi^6*δ^2*λ^6*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^8)
    end

    if dorder==[3, 0, 0, 2]
        func = 192*pi^3*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 128*pi^4*δ*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 128*pi^4*δ*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 - 64*pi^5*δ^2*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^7 - 576*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 576*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 384*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 384*pi^4*δ*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 768*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 2816*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*pi^5*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 960*pi^5*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^7) - 3072*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 3072*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 960*pi^5*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) + 4480*pi^5*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 960*pi^5*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^7) - 2048*pi^4*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*pi^5*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) + 2560*pi^5*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^7) - 1024*pi^5*δ^2*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^7)
    end

    if dorder==[2, 0, 0, 2]
        func = 24*pi^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 24*pi^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*pi^4*δ^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 16*pi^4*δ^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 96*pi^3*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 96*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 48*pi^4*δ^2*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 288*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 288*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 352*pi^4*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 384*pi^3*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*pi^4*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 384*pi^4*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 256*pi^4*δ^2*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6)
    end

    if dorder==[1, 0, 0, 2]
        func = -8*pi*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*pi^2*δ*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*pi^2*δ*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^3*δ^2*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*pi^2*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^3*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*pi^3*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 64*pi^3*δ^2*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5)
    end

    if dorder==[0, 0, 0, 2]
        func = 4*pi^2*δ^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 4*pi^2*δ^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 8*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*pi^2*δ^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4)
    end

    if dorder==[4, 0, 1, 1]
        func = 64*pi^4*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*pi^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 + 64*pi^5*δ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 + 512*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*pi^4*λ*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 2944*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 4992*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 7424*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1920*pi^5*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 2048*pi^5*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 7168*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 3072*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 2880*pi^5*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 15360*pi^5*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 4800*pi^5*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 2048*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^5) + 12160*pi^5*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 19200*pi^5*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1920*pi^5*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^7*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 10240*pi^5*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 5120*pi^5*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^7*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) + 2048*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*sin(pi*δ/rq_P)^7*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^6)
    end

    if dorder==[3, 0, 1, 1]
        func = -48*pi^3*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*δ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*pi^4*δ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 + 288*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 384*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 128*pi^4*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 96*pi^4*δ*λ*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 864*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 288*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 96*pi^4*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 736*pi^4*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*pi^3*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1248*pi^4*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1856*pi^4*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1792*pi^4*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 768*pi^4*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 512*pi^4*δ*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^5)
    end

    if dorder==[2, 0, 1, 1]
        func = -8*pi^2*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*pi^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*pi^3*δ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*pi^2*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 80*pi^2*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 64*pi^2*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*pi^3*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 128*pi^3*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 288*pi^3*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*pi^3*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 128*pi^3*δ*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if dorder==[1, 0, 1, 1]
        func = 4*pi*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*δ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*pi^2*δ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*pi*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 8*pi^2*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 40*pi^2*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 32*pi^2*δ*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if dorder==[0, 0, 1, 1]
        func = 4*pi*δ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 + 8*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if dorder==[4, 1, 0, 1]
        func = -192*pi^4*λ^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 192*pi^4*λ^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 64*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*pi^4*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 192*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 192*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) + 1408*pi^4*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 1408*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 1536*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1536*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 1536*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 48*pi^4*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 896*pi^5*δ*λ^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 896*pi^5*δ*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 64*pi^5*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 - 3072*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) + 3072*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) - 1024*pi^4*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1024*pi^4*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1024*pi^4*λ^4*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 1408*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 352*pi^4*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 896*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^6) - 896*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^6) - 960*pi^5*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 4480*pi^5*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 960*pi^5*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 768*pi^5*δ*λ^3*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 3072*pi^5*δ*λ^3*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 768*pi^5*δ*λ^3*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^5) + 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-3 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-3 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-3 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-3 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1728*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) + 7552*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) - 1728*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^6) - 2560*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 2560*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*pi^5*δ*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 2560*pi^5*δ*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1024*pi^5*δ*λ^4*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*pi^5*δ*λ^4*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 896*pi^5*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 896*pi^5*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) - 64*pi^5*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 64*pi^5*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 1024*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-4 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-4 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))/(α^3*rq_P^4) - 6144*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^6) + 6144*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^6) - 1024*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 1024*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) - 1024*pi^5*δ*λ^5*(-4 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6) + 768*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 768*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 192*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 1408*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 192*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 3072*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^5*rq_P^6) + 1024*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1024*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1536*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 1536*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6)
    end

    if dorder==[3, 1, 0, 1]
        func = -144*pi^3*λ^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 144*pi^3*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^3*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^3*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 48*pi^4*δ*λ^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 48*pi^4*δ*λ^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 16*pi^4*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*pi^4*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 144*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 144*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 192*pi^3*λ^3*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 192*pi^3*λ^3*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 48*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 48*pi^4*δ*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) + 288*pi^4*δ*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 384*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 144*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 144*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^3*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 288*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^5) - 384*pi^4*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 384*pi^4*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*pi^4*δ*λ^3*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*pi^4*δ*λ^3*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 48*pi^4*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*pi^4*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*pi^3*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-3 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-3 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 576*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) + 576*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^5) - 256*pi^4*δ*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 256*pi^4*δ*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) - 288*pi^4*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 64*pi^4*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) - 512*pi^4*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^5) + 192*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 192*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 192*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 256*pi^4*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5)
    end

    if dorder==[2, 1, 0, 1]
        func = -8*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*pi^2*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 32*pi^2*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 32*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 32*pi^3*δ*λ^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 32*pi^3*δ*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*pi^3*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 32*pi^2*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 16*pi^2*λ^2*(-1 - α)*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-2 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 32*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 32*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 64*pi^3*δ*λ^3*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 64*pi^3*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 32*pi^3*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 32*pi^3*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 16*pi^3*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*pi^3*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 64*pi^3*δ*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4)
    end

    if dorder==[1, 1, 0, 1]
        func = 4*pi*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi*λ*(4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 8*pi*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)/(α^3*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*pi*δ*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)/(α^2*rq_P^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P - 4*pi^2*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*pi^2*δ*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*pi^2*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3)
    end

    if dorder==[0, 1, 0, 1]
        func = 8*pi*δ*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) + 4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(2*λ*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2
    end

    if dorder==[4, 0, 0, 1]
        func = 64*pi^4*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 64*pi^4*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 + 192*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 64*pi^5*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^6 - 1408*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 960*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 960*pi^5*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^6) + 1024*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5) + 960*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) - 4480*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 960*pi^5*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^6) + 2560*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) - 2560*pi^5*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^6) + 1024*pi^5*δ*λ^5*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^5*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^6)
    end

    if dorder==[3, 0, 0, 1]
        func = -48*pi^3*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*δ*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 - 16*pi^4*δ*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^5 + 144*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 144*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 48*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 48*pi^4*δ*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 192*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 352*pi^4*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^5) + 384*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) - 384*pi^4*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^5) + 256*pi^4*δ*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^5)
    end

    if dorder==[2, 0, 0, 1]
        func = -8*pi^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 8*pi^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 16*pi^3*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 32*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^3*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 64*pi^3*δ*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4)
    end

    if dorder==[1, 0, 0, 1]
        func = 4*pi*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*δ*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 4*pi^2*δ*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*pi^2*δ*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3)
    end

    if dorder==[0, 0, 0, 1]
        func = 4*pi*δ*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2
    end

    if dorder==[4, 0, 2, 0]
        func = -160*pi^4*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*pi^4*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*pi^4*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 448*pi^4*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 192*pi^4*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^8*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 2688*pi^4*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 5184*pi^4*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 7872*pi^4*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 6016*pi^4*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 5632*pi^4*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 1536*pi^4*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^8*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) - 1024*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-5 - α)*sin(pi*δ/rq_P)^8*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-6 - α)/(α^5*rq_P^4)
    end

    if dorder==[3, 0, 2, 0]
        func = -96*pi^3*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 160*pi^3*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 768*pi^3*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 448*pi^3*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 960*pi^3*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) + 192*pi^3*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^7*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) - 256*pi^3*λ^3*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^7*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^3)
    end

    if dorder==[2, 0, 2, 0]
        func = 16*pi^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 48*pi^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 16*pi^2*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) - 144*pi^2*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) - 64*pi^2*λ^2*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^2)
    end

    if dorder==[1, 0, 2, 0]
        func = -16*pi*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P) - 16*pi*λ*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P)
    end

    if dorder==[0, 0, 2, 0]
        func = -4*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/α
    end

    if dorder==[4, 1, 1, 0]
        func = -16*pi^4*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 96*pi^4*λ*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*pi^4*λ*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*pi^4*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P^4 + 16*pi^4*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P^4 + 96*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*pi^4*λ*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*pi^4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 704*pi^4*λ*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 704*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 1248*pi^4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1856*pi^4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1152*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1152*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*pi^4*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*pi^4*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 32*pi^4*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^4) + 2400*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 3008*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1792*pi^4*λ^3*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*pi^4*λ^3*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1792*pi^4*λ^3*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*pi^4*λ^3*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 1024*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 96*pi^4*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 704*pi^4*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 32*pi^4*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^4) + 4608*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) + 512*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 512*pi^4*λ^4*(-4 - α)*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) + 512*pi^4*λ^4*(-4 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4) - 1152*pi^4*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1152*pi^4*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 96*pi^4*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 704*pi^4*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α^2*rq_P^4) + 1536*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^5*rq_P^4) - 1024*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 768*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 768*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-4 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α^3*rq_P^4) - 512*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4)
    end

    if dorder==[3, 1, 1, 0]
        func = 16*pi^3*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 96*pi^3*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 96*pi^3*λ*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 16*pi^3*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P^3 + 96*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 96*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 288*pi^3*λ^2*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 96*pi^3*λ^2*(-2 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 192*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 480*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 96*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) + 128*pi^3*λ^3*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) + 128*pi^3*λ^3*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3) - 96*pi^3*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 96*pi^3*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 32*pi^3*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^3) + 256*pi^3*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^3) - 192*pi^3*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 96*pi^3*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*pi^3*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α^2*rq_P^3) - 128*pi^3*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3)
    end

    if dorder==[2, 1, 1, 0]
        func = 4*pi^2*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 + 4*pi^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P^2 - 4*pi^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P^2 + 32*pi^2*λ*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*pi^2*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) + 32*pi^2*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2) + 8*pi^2*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*pi^2*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^2) - 32*pi^2*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 8*pi^2*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 16*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/(α*rq_P^2) - 32*pi^2*λ^2*(-2 - α)*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2)
    end

    if dorder==[1, 1, 1, 0]
        func = -4*pi*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P - 4*pi*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)*(-2*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 2*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^4/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2))/rq_P - 8*pi*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P)
    end

    if dorder==[0, 1, 1, 0]
        func = -2*sin(pi*δ/rq_P)^2*(2*λ*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α) - 4*λ*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/α^2
    end

    if dorder==[4, 0, 1, 0]
        func = -16*pi^4*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 128*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*pi^4*λ*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 96*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 736*pi^4*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 1248*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1856*pi^4*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 1792*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 768*pi^4*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*pi^4*λ^4*(-4 - α)*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^6*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-5 - α)/(α^4*rq_P^4)
    end

    if dorder==[3, 0, 1, 0]
        func = 16*pi^3*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 96*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 128*pi^3*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 288*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 96*pi^3*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 128*pi^3*λ^3*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^5*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^3)
    end

    if dorder==[2, 0, 1, 0]
        func = 4*pi^2*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 + 8*pi^2*λ*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 40*pi^2*λ*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 32*pi^2*λ^2*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^2)
    end

    if dorder==[1, 0, 1, 0]
        func = -4*pi*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P - 8*pi*λ*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P)
    end

    if dorder==[0, 0, 1, 0]
        func = -2*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)
    end

    if dorder==[4, 2, 0, 0]
        func = -96*pi^4*λ^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 96*pi^4*λ^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 16*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 16*pi^4*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 96*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) - 96*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) + 704*pi^4*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 768*pi^4*λ^3*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 768*pi^4*λ^3*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 96*pi^4*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 96*pi^4*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 704*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^4) - 1536*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1536*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 1536*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 1536*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 512*pi^4*λ^4*(-3 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*pi^4*λ^4*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 512*pi^4*λ^4*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 96*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 96*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 704*pi^4*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 2304*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^4) + 2304*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^4) - 1536*pi^4*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*pi^4*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 1536*pi^4*λ^4*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 704*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 768*pi^4*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 768*pi^4*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 768*pi^4*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 768*pi^4*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 352*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*pi^4*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 3072*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^5*rq_P^4) + 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 1536*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 512*pi^4*λ^4*(-3 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 512*pi^4*λ^4*(-3 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 512*pi^4*λ^4*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 1536*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-4 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if dorder==[3, 2, 0, 0]
        func = -96*pi^3*λ^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 96*pi^3*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 128*pi^3*λ^3*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 16*pi^3*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 16*pi^3*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 96*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^3) + 96*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^3) - 256*pi^3*λ^3*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 256*pi^3*λ^3*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) + 96*pi^3*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 96*pi^3*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 384*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^4*rq_P^3) + 96*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 96*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 128*pi^3*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 128*pi^3*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 48*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*pi^3*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 256*pi^3*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-3 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if dorder==[2, 2, 0, 0]
        func = 4*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 + 4*pi^2*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 32*pi^2*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 32*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^3*rq_P^2) + 32*pi^2*λ^2*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 32*pi^2*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 16*pi^2*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) - 16*pi^2*λ^2*(-1 - α)*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-2 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if dorder==[1, 2, 0, 0]
        func = -4*pi*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P - 4*pi*λ*(4*λ*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - 4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4/(α^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^2) + 4*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if dorder==[0, 2, 0, 0]
        func = (2*λ*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-α) + 4*λ^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/α^3
    end

    if dorder==[4, 1, 0, 0]
        func = 48*pi^4*λ^2*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 48*pi^4*λ^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 16*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 48*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 48*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) - 352*pi^4*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 352*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 384*pi^4*λ^3*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) - 768*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^4) + 256*pi^4*λ^4*(-3 - α)*(-2 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 256*pi^4*λ^4*(-3 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 256*pi^4*λ^4*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4) + 352*pi^4*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 768*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^4*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*(-2*λ*(-4 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if dorder==[3, 1, 0, 0]
        func = 48*pi^3*λ^2*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 48*pi^3*λ^2*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 16*pi^3*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 + 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) - 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^3) + 64*pi^3*λ^3*(-2 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 64*pi^3*λ^3*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3) + 128*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^3*rq_P^3) - 48*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*(-2*λ*(-3 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if dorder==[2, 1, 0, 0]
        func = 4*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 + 16*pi^2*λ^2*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2) + 16*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α^2*rq_P^2) - 16*pi^2*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if dorder==[1, 1, 0, 0]
        func = -4*pi*λ*(-2*λ*(-1 - α)*sin(pi*δ/rq_P)^2/(α^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if dorder==[0, 1, 0, 0]
        func = (2*λ*sin(pi*δ/rq_P)^2/(α*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)) - log(1 + 2*λ*sin(pi*δ/rq_P)^2/α))*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-α)
    end

    if dorder==[4, 0, 0, 0]
        func = -16*pi^4*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 + 16*pi^4*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^4 - 48*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 48*pi^4*λ^2*(-1 - α)*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) + 352*pi^4*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^4) - 384*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) + 384*pi^4*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^4) - 256*pi^4*λ^4*(-3 - α)*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^4*cos(pi*δ/rq_P)^4*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-4 - α)/(α^3*rq_P^4)
    end

    if dorder==[3, 0, 0, 0]
        func = 16*pi^3*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^3 - 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) + 48*pi^3*λ^2*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^3) - 64*pi^3*λ^3*(-2 - α)*(-1 - α)*sin(pi*δ/rq_P)^3*cos(pi*δ/rq_P)^3*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-3 - α)/(α^2*rq_P^3)
    end

    if dorder==[2, 0, 0, 0]
        func = 4*pi^2*λ*sin(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 4*pi^2*λ*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P^2 - 16*pi^2*λ^2*(-1 - α)*sin(pi*δ/rq_P)^2*cos(pi*δ/rq_P)^2*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-2 - α)/(α*rq_P^2)
    end

    if dorder==[1, 0, 0, 0]
        func = -4*pi*λ*sin(pi*δ/rq_P)*cos(pi*δ/rq_P)*(1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-1 - α)/rq_P
    end

    if dorder==[0, 0, 0, 0]
        func = (1 + 2*λ*sin(pi*δ/rq_P)^2/α)^(-α)
    end

    return even_time_derivative * float(func)  # correcting for amount of t2 derivatives

end


return rq_periodic_kernel, 3  # the function handle and the number of kernel hyperparameters
